﻿using System;
using System.Linq;
using System.Activities;
using System.Activities.Statements;
using Microsoft.Security.Activities;
using System.Security.Cryptography.X509Certificates;
using System.IdentityModel.Tokens;
using System.ComponentModel;
using Microsoft.Activities.SecurityPack.Designers;

namespace X509SecurityTokenSample
{
    [Designer(typeof(GetX509SecurityTokenDesigner))]
    public class GetX509SecurityToken : GetSecurityToken
    {
        public GetX509SecurityToken()
        {
            FindType = X509FindType.FindBySubjectName;
            StoreLocation = StoreLocation.CurrentUser;
            StoreName = StoreName.My;
        }

        public InArgument<X509Certificate2> Certificate { get; set; }
        public X509FindType FindType { get; set; }
        public StoreLocation StoreLocation { get; set; }
        public InArgument<string> FindValue { get; set; }
        public StoreName StoreName { get; set; }

        protected override void Execute(NativeActivityContext context)
        {
            X509Certificate2 targetCert = null;
            if (this.Certificate != null)
                targetCert = this.Certificate.Get(context);
            if (targetCert == null)
            {
                var store = new X509Store(StoreName, StoreLocation);
                try
                {
                    store.Open(OpenFlags.ReadOnly);
                    var col = store.Certificates.Find(FindType, FindValue.Get(context), false);
                    if (col.Count > 0)
                        targetCert = col[0];//Use first certificate mathing the search criteria
                }
                finally
                {
                    if (store != null)
                        store.Close();
                }
            }
            if (targetCert == null)
                throw new InvalidOperationException("No certificate found using the specified find criteria.");
            // enlist the token as a flow token
            base.EnlistSecurityToken(context, new X509SecurityToken(targetCert));
        }
    }
}
