﻿using System;
using System.Security.Cryptography.X509Certificates;
using System.ServiceModel;
using Microsoft.IdentityModel.Claims;
using Microsoft.IdentityModel.Configuration;
using Microsoft.IdentityModel.Protocols.WSTrust;
using Microsoft.IdentityModel.SecurityTokenService;



namespace TestWorkflowService
{
    class CustomSecurityTokenService : SecurityTokenService
    {
        public CustomSecurityTokenService(SecurityTokenServiceConfiguration config)
            : base(config)        {   
        }

        protected override Scope GetScope(IClaimsPrincipal principal, RequestSecurityToken request)
        {
            var appliesTo = request.AppliesTo.Uri.AbsoluteUri;
            Scope scope = new Scope(appliesTo, SecurityTokenServiceConfiguration.SigningCredentials);

            scope.ReplyToAddress = request.ReplyTo;
            if (string.IsNullOrEmpty(scope.ReplyToAddress))
                scope.ReplyToAddress = request.AppliesTo.Uri.AbsoluteUri;

            scope.EncryptingCredentials = new X509EncryptingCredentials(GetRPCert(request.AppliesTo));
            return scope;
        }

        private static X509Certificate2 GetRPCert(EndpointAddress appliesTo)
        {
            switch(appliesTo.Uri.AbsoluteUri)
            {
                default:
                    return new X509Certificate2("localhost.cer");
            }
        }

        void ValidateAppliesTo(EndpointAddress appliesTo)
        {
            if (appliesTo == null)
            {
                throw new InvalidRequestException("The appliesTo is null.");
            }
        }

        public override RequestSecurityTokenResponse Issue(IClaimsPrincipal principal, RequestSecurityToken request)
        {
            Console.WriteLine("Issuing token...");
            var token = base.Issue(principal, request);
            return token;
        }

        protected override IClaimsIdentity GetOutputClaimsIdentity(IClaimsPrincipal principal, RequestSecurityToken request, Scope scope)
        {
             ValidateAppliesTo(request.AppliesTo);

            // Create new identity and copy content of the caller's identity into it (including the existing delegate chain)
            IClaimsIdentity callerIdentity = (IClaimsIdentity)principal.Identity;
            IClaimsIdentity outputIdentity = callerIdentity.Copy();

            // If there is an ActAs token in the RST, return a copy of it as the top-most identity
            // and put the caller's identity into the Actor property of this identity.
            if (request.ActAs != null)
            {
                IClaimsIdentity actAsSubject = request.ActAs.GetSubject()[0];
                IClaimsIdentity actAsIdentity = actAsSubject.Copy();

                // Find the last actor in the actAs identity
                IClaimsIdentity lastActor = actAsIdentity;
                while (lastActor.Actor != null)
                {
                    lastActor = lastActor.Actor;
                }

                // Set the caller's identity as the last actor in the delegation chain
                lastActor.Actor = outputIdentity;

                // Return the actAsIdentity instead of the caller's identity in this case
                outputIdentity = actAsIdentity;
                Console.WriteLine("Creating ActAs claims...");
            }
            if (outputIdentity.Claims.Count == 0) 
                outputIdentity.Claims.AddRange(new Claim[]
                {
                    new Claim(System.IdentityModel.Claims.ClaimTypes.Name, "Bob Creamer"),
                    new Claim(System.IdentityModel.Claims.ClaimTypes.Email, "bobc@fabrikam.com"),
                    new Claim(System.IdentityModel.Claims.ClaimTypes.Upn, "bobc@fabrikam.com"),
                });
            return outputIdentity;
        }
    }
}