﻿using System;
using System.IdentityModel.Selectors;
using System.Security.Cryptography.X509Certificates;
using System.ServiceModel.Security;
using Extensions.Utilities;
using Microsoft.IdentityModel.Configuration;
using Microsoft.IdentityModel.Protocols.WSTrust;
using Microsoft.IdentityModel.SecurityTokenService;
using Microsoft.IdentityModel.Tokens;

namespace BasicSTS
{


    class Program
    {
        static void Main(string[] args)
        {
            X509Certificate2 stsSignCert = 
                new X509Certificate2("BasicSts.pfx","a");

            var stsConfig = new SecurityTokenServiceConfiguration
            {
                ServiceCertificate = stsSignCert,
                SigningCredentials= new X509SigningCredentials(stsSignCert),
                SecurityTokenService = typeof(CustomSecurityTokenService),
                TokenIssuerName = "urn:zamdBasicSts",
            };
            stsConfig.SecurityTokenHandlers.AddOrReplace(new CustomUserNameSecurityTokenHandler
            {
                UserNamePasswordValidator = (uid, pwd) => { Console.WriteLine("{0} authenticated.", uid); }
            }
            );
            var actAsConfig = new ServiceConfiguration
            {
                AudienceRestriction = new AudienceRestriction(AudienceUriMode.Never),
                CertificateValidationMode = X509CertificateValidationMode.None,
                IssuerNameRegistry = new MyTestRegistry()
            };
            actAsConfig.Initialize();

            stsConfig.SecurityTokenHandlerCollectionManager[SecurityTokenHandlerCollectionManager.Usage.ActAs] = actAsConfig.SecurityTokenHandlers;

            var stsHost = new WSTrustServiceHost(stsConfig);
            stsHost.Open();


            Console.WriteLine(stsHost.WriteEndpoints());
            Console.ReadLine();

            stsHost.Close();

        }
    }




    public class MyTestRegistry : IssuerNameRegistry
    {
        public override string GetIssuerName(System.IdentityModel.Tokens.SecurityToken securityToken)
        {
            return "TestIssuer";
        }
    }
}
