﻿using System;
using System.Linq;
using System.Security.Cryptography.X509Certificates;
using System.ServiceModel.Activities;
using Extensions.Utilities;
using Microsoft.IdentityModel.Claims;
using Microsoft.IdentityModel.Configuration;
using Microsoft.IdentityModel.Tokens;

namespace SampleService
{
    class Program
    {
        static void Main(string[] args)
        {
            var wf = new Workflow1();

            var wfsh = new WorkflowServiceHost(wf, new Uri("http://localhost:8080/Echo"));

            var configuration = new ServiceConfiguration
            {
                ServiceCertificate = new X509Certificate2("SampleService.pfx", "a"),
                IssuerNameRegistry = new MyTestRegistry(),
                CertificateValidationMode =  System.ServiceModel.Security.X509CertificateValidationMode.None,
                ClaimsAuthorizationManager = new SimpleAuthorizationManager()
            };
            configuration.AudienceRestriction.AllowedAudienceUris.Add(wfsh.BaseAddresses.First() );
            FederatedServiceCredentials.ConfigureServiceHost(wfsh, configuration);
            wfsh.Open();

            Console.WriteLine(wfsh.WriteEndpoints());

            Console.ReadLine();
            wfsh.Close();
        }



    }

    public class MyTestRegistry : IssuerNameRegistry
    {
        public override string GetIssuerName(System.IdentityModel.Tokens.SecurityToken securityToken)
        {
            return "TestIssuer";
        }
    }


    class SimpleAuthorizationManager : ClaimsAuthorizationManager
    {
        public override bool CheckAccess(AuthorizationContext context)
        {
            PrintCallerIdentity((IClaimsIdentity)context.Principal.Identity);
            return true;    
        }

        void PrintCallerIdentity(IClaimsIdentity identity)
        {
            Console.ForegroundColor = ConsoleColor.Green;
            Console.WriteLine("Caller's name: " + identity.Name);
            foreach (Claim claim in identity.Claims)
            {
                Console.WriteLine("ClaimType  : " + claim.ClaimType.Split('/').Last());
                Console.WriteLine("ClaimValue : " + claim.Value);
                Console.WriteLine();
            }
            Console.ResetColor();
            while (identity.Actor != null)
            {
                Console.ForegroundColor = ConsoleColor.Yellow;
                Console.WriteLine("Acting Via: " + identity.Actor.Name);
                foreach (Claim claim in identity.Actor.Claims)
                {
                    Console.WriteLine("ClaimType  : " + claim.ClaimType.Split('/').Last());
                    Console.WriteLine("ClaimValue : " + claim.Value);
                    Console.WriteLine();
                }
                identity = identity.Actor;
                Console.ResetColor();
            }

            Console.WriteLine();
        }
        
    }
}
