﻿using System;
using System.IdentityModel.Tokens;
using System.Linq;
using System.Security.Cryptography.X509Certificates;
using System.ServiceModel.Activities;
using System.ServiceModel.Security;
using Extensions.Utilities;
using Microsoft.IdentityModel.Configuration;
using Microsoft.IdentityModel.Tokens;
namespace WorkflowServiceFacade
{

    class Program
    {
        static void Main(string[] args)
        {
            var wfsh = new WorkflowServiceHost(new WFServiceFacade(), new Uri("http://localhost:8000/WFFacade"));

            var configuration = new ServiceConfiguration
            {
                ServiceCertificate = new X509Certificate2("WFServiceFacade.pfx", "a"),
                CertificateValidationMode = X509CertificateValidationMode.None,
                SaveBootstrapTokens = true,
                IssuerNameRegistry = new CertBasedIssuerNameRegistry()
            };
            configuration.AudienceRestriction.AllowedAudienceUris.Add(wfsh.BaseAddresses.First());
            FederatedServiceCredentials.ConfigureServiceHost(wfsh, configuration);

            wfsh.Open();
            Console.WriteLine(wfsh.WriteEndpoints());
            Console.ReadLine();
            wfsh.Close();
        }
    }

    class CertBasedIssuerNameRegistry : IssuerNameRegistry
    {
        public override string GetIssuerName(SecurityToken securityToken)
        {
            var cert = securityToken as X509SecurityToken;
            if (cert != null)
            {
                return cert.Certificate.Issuer;
            }
            return null;
        }
    }

}
